@@ -1,3 +1,4 @@
+import csv
 import ctypes
 import _ctypes
 import hashlib
@@ -8,9 +9,11 @@ import os
 import sys
 import logging
 
-import importlib.util as import_util
+import importlib
+import importlib.util
 from ctypes import util
 from os import path
+from numbers import Number
 
 DEFAULT_MAX_READ = 1024 * 1024
 
@@ -106,12 +109,10 @@ def write_to_file(lines, filename):
         my_file.close()
 
 
-def load_module(filename, module_name=None):
-    if not module_name:
-        module_name = 'unnamed_module.' + path.basename(path.splitext(filename)[0])
-
-    spec = import_util.spec_from_file_location(module_name, filename)
-    module = import_util.module_from_spec(spec)
+def load_module(filename, package_name=None):
+    module_name = (package_name if package_name else '') + path.basename(path.splitext(filename)[0])
+    spec = importlib.util.spec_from_file_location(module_name, filename)
+    module = importlib.util.module_from_spec(spec)
 
     try:
         spec.loader.exec_module(module)
@@ -284,3 +285,65 @@ def convert_curses_capture(capture):
             line_set.append(line[next:])
         converted[line_no] = "".join(line_set)
     return converted
+
+
+def causes_exception(test_me):
+    try:
+        test_me()
+    except:
+        return True
+    return False
+
+
+def can_write_property(target, property_name, test_value):
+    return not causes_exception(lambda: exec("target." + property_name + " = test_value"))
+
+
+def get_public_attr(classtype, exclusions=[], filter_callable=True):
+    key_list = [key for key in classtype.__dict__.keys() if not key.startswith("_")] # Not protected/private
+    key_list = [key for key in key_list if not type(classtype.__dict__[key]).__name__ in exclusions] # Not excluded
+    if filter_callable:
+        key_list = [key for key in key_list if not callable(classtype.__dict__[key])] # Keep non-callable
+    return key_list
+
+
+def get_public_vars(target, filter_callable=True):
+    var_list = [entry for entry in vars(target).keys() if not entry.startswith("_")]
+    if filter_callable:
+        var_list = [key for key in var_list if not callable(vars(target)[key])]
+    return var_list
+
+
+def save_csv(filename, row_data, insert_key = False, dialect = 'excel'):
+    # Open file for writing
+    with open(filename, 'w', newline = '') as csvFile:
+        writer = csv.writer(csvFile, dialect, quotechar='"', delimiter=',')
+
+        # Write the data
+        if insert_key:
+            for key, row in row_data.iteritems():
+                print("Key: " + key + " Value: " + row)
+                writer.writerow('{:f}'.format(val) if isinstance(val, Number) else val for val in [ key ] + row)
+        else:
+            for row in row_data:
+                writer.writerow('{:f}'.format(val) if isinstance(val, Number) else val for val in row)
+
+        csvFile.close()
+
+
+def append_csv(filename, row_data, insert_key = False, dialect = 'excel'):
+    # Open file for appending
+    with open(filename, 'a', newline = '') as csvFile:
+        writer = csv.writer(csvFile, dialect, quotechar='"', delimiter=',')
+
+        # Write the data
+        if insert_key:
+            for key, row in row_data.iteritems():
+                print("Key: " + key + " Value: " + row)
+                writer.writerow('{:f}'.format(val) if isinstance(val, Number) else val for val in [ key ] + row)
+        else:
+            for row in row_data:
+                writer.writerow('{:f}'.format(val) if isinstance(val, Number) else val for val in row)
+
+        csvFile.close()
+
